@@ -15,13 +15,17 @@ YOUTUBE = "https://youtube.com/watch?v="
 
 DOWNLOAD_SUCCESS_MSG = "Download successfull: {}"
 DOWNLOADING_MSG = "Downloading {}: {}"
+DOWNLOADING_ERROR_MSG = "Downloading Error {}: {}"
 INVALID_URL_MSG = "ERROR: {} is not a valid url"
 MESSAGES = [
     DOWNLOAD_SUCCESS_MSG,
     INVALID_URL_MSG,
-    DOWNLOADING_MSG
+    DOWNLOADING_MSG,
+    DOWNLOADING_ERROR_MSG,
 ]
 
+current_course = ""
+
 
 def print_message(status: int, messages: list[str]):
     """
@@ -30,6 +34,14 @@ def print_message(status: int, messages: list[str]):
     print(MESSAGES[status].format(*messages))
 
 
+def logerror(status: int, messages: list[str]):
+    """
+    Log error message from message list depending on status.
+    """
+    with open("nptel_dl.err", "a") as f:
+        f.write(f"{current_course}: " + MESSAGES[status].format(*messages) + "\n")
+
+
 def write_json(dict_: dict, filename: str):
     with open(filename, "w") as f:
         f.write(json.dumps(dict_, indent=4))
@@ -37,7 +49,7 @@ def write_json(dict_: dict, filename: str):
 
 def get_dict(dict_: dict, dict_name: str):
     """
-    Return Dict value for a key or entire dict if key=all 
+    Return Dict value for a key or entire dict if key=all
     """
     if dict_name == "all":
         return dict_
@@ -154,7 +166,11 @@ def get_course_dict(course_data: tuple[str, str, str]):
             video = video_tag.get("onclick")[17:]
             video_id, yt_video_id, direct_video_link = eval(video)
             yt_video_link = f"{YOUTUBE}{yt_video_id}" if yt_video_id else ""
-            direct_video_link = f"{ARCHIVE_WEBSITE}{direct_video_link}" if direct_video_link else yt_video_link
+            direct_video_link = (
+                f"{ARCHIVE_WEBSITE}{direct_video_link}"
+                if direct_video_link and "livession" not in direct_video_link
+                else yt_video_link
+            )
             videos_dict = {
                 "video_id": video_id,
                 "video_title": video_title,
@@ -173,7 +189,11 @@ def get_course_dict(course_data: tuple[str, str, str]):
                 video = video_tag.get("onclick")[17:]
                 video_id, yt_video_id, direct_video_link = eval(video)
                 yt_video_link = f"{YOUTUBE}{yt_video_id}" if yt_video_id else ""
-                direct_video_link = f"{ARCHIVE_WEBSITE}{direct_video_link}" if direct_video_link else yt_video_link
+                direct_video_link = (
+                    f"{ARCHIVE_WEBSITE}{direct_video_link}"
+                    if direct_video_link and "livession" not in direct_video_link
+                    else yt_video_link
+                )
                 videos_dict = {
                     "video_id": video_id,
                     "video_title": video_title,
@@ -238,7 +258,7 @@ def write_jsons(input_urls: list[str], dict_name: str, single: bool):
     course_ids = []
     for input_url, course_dict in courses:
         if not course_dict:
-            print_message(1, [ input_url ])
+            print_message(1, [input_url])
             continue
         course_id = course_dict["course_id"]
         output_dict = get_dict(course_dict, dict_name)
@@ -257,35 +277,47 @@ def ytdl_download(link: str, ytdl_opts: dict):
     """
     Download link using yt-dlp
     """
+    print_message(2, [ytdl_opts["outtmpl"], link])
     if not link:
         return
     link = link.replace(WEBSITE, ARCHIVE_WEBSITE)
-    with YoutubeDL(ytdl_opts) as ytdl:
-        ytdl.download(link)
+    try:
+        with YoutubeDL(ytdl_opts) as ytdl:
+            ytdl.download(link)
+    except:
+        print_message(3, [ytdl_opts["outtmpl"], link])
+        logerror(3, [ytdl_opts["outtmpl"], link])
 
 
-def drive_download(link: str, file_path: str="", filename: str=""):
+def drive_download(link: str, file_path: str = "", filename: str = ""):
     """
     Download google drive links
     """
+    print_message(2, [filename, link])
     if not link:
         return
     link = link.replace(WEBSITE, ARCHIVE_WEBSITE)
     os.makedirs(file_path, exist_ok=True)
     output = f"{file_path}/{filename}" if filename else f"{file_path}/"
-    gdown.download(link, output, fuzzy=True)
+    try:
+        gdown.download(link, output, fuzzy=True)
+    except:
+        print_message(3, [filename, link])
+        logerror(3, [f"{file_path}/{filename}", link])
 
 
 def download_course(input_url: str, opts: dict) -> int:
     """
     Download course from url or course id
     """
+    global current_course
     course_data = get_course_data(input_url)
     required_list = opts["required"]
     if course_data is None:
         return 1
     course_dict = get_course_dict(course_data)
     course_id = course_dict["course_id"]
+    current_course = course_id
     course_title = course_dict["course_title"]
     course_dir = f"{course_id}. {course_title}"
     ytdl_opts = {}
@@ -293,39 +325,39 @@ def download_course(input_url: str, opts: dict) -> int:
         if required == "all":
             ytdl_opts["outtmpl"] = f"{course_dir}/syllabus.pdf"
             syllabus = get_dict(course_dict, "syllabus")
-            print_message(2, ["syllabus", syllabus])
             ytdl_download(syllabus, ytdl_opts)
             for required_ in ["books", "assignements"]:
                 required_dict = get_dict(course_dict, required_)
                 for name, link in required_dict.items():
-                    print_message(2, [name, link])
                     ytdl_opts["outtmpl"] = f"{course_dir}/{required_}/{name}.pdf"
                     ytdl_download(link, ytdl_opts)
             modules = get_dict(course_dict, "modules")
             for module in modules:
                 module_title, videos = module.values()
                 for video in videos:
-                    video_id, video_title, yt_video_link, direct_video_link, transcript_link = video.values()
+                    (
+                        video_id,
+                        video_title,
+                        yt_video_link,
+                        direct_video_link,
+                        transcript_link,
+                    ) = video.values()
                     filename = f"{video_id}. {video_title}.%(ext)s"
                     ytdl_opts["outtmpl"] = f"{course_dir}/{module_title}/{filename}"
-                    print_message(2, [filename, direct_video_link])
                     ytdl_download(direct_video_link, ytdl_opts)
                     filename = f"{video_id}. {video_title}.pdf"
                     file_path = f"{course_dir}/{module_title}"
-                    print_message(2, [filename, transcript_link])
                     drive_download(transcript_link, file_path, filename)
-                    
+
         if required == "syllabus":
             ytdl_opts["outtmpl"] = f"{course_dir}/syllabus.pdf"
             syllabus = get_dict(course_dict, "syllabus")
-            print_message(2, ["syllabus", syllabus])
             ytdl_download(syllabus, ytdl_opts)
         if required in ["books", "transcripts", "assignements"]:
             required_dict = get_dict(course_dict, required)
             for name, link in required_dict.items():
                 file_path = f"{course_dir}/{required}"
-                filename = f"{name}.pdf" 
-                print_message(2, [filename, link])
+                filename = f"{name}.pdf"
                 if "drive.google.com" in link:
                     drive_download(link, file_path, filename)
                     continue
@@ -340,7 +372,7 @@ def download_courses(input_urls: list[str], opts: dict):
     """
     for input_url in input_urls:
         download_status = download_course(input_url, opts)
-        print_message(download_status, [ input_url ])
+        print_message(download_status, [input_url])
 
 
 def main():
